package net.wanji.opt.servicev2.judgeanalysis.impl;

import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisGreenCongestionPeriodMapper;
import net.wanji.opt.entity.judgeanalysis.AnalysisGreenCongestionPeriod;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisGreenCongestionPeriodService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.temporal.TemporalAdjusters;
import java.time.temporal.WeekFields;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 干线拥堵时段实现类
 * </p>
 *
 * @author hwm
 * @since 2025-03-20
 */
@Service
public class AnalysisGreenCongestionPeriodServiceImpl implements AnalysisGreenCongestionPeriodService {
    @Resource
    private AnalysisGreenCongestionPeriodMapper analysisGreenCongestionPeriodMapper;

    public void selectCountByCongestionPeriod() throws ParseException {
        LocalDate currentDate = LocalDate.now();
        // 获取本周一的日期
        LocalDate mondayOfThisWeek = currentDate.with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
        // 获取上周一的日期
        LocalDate todayOfWeek = mondayOfThisWeek.plusDays(-7);
        //获取上周日的日期
        LocalDate endOfWeek = mondayOfThisWeek.plusDays(-1);
        WeekFields weekFields = WeekFields.of(Locale.getDefault());
        //获取上周的周数
        int Number = todayOfWeek.get(weekFields.weekOfYear());
        int year = currentDate.getYear();
        String weekNumber = String.valueOf(year) + String.valueOf(Number);
        //循环7天处理数据
        for (int i = 0; i < 7; i++) {
            LocalDate date = todayOfWeek.plusDays(i);
            //获取干线数据
            List<AnalysisGreenCongestionPeriod> greenList = analysisGreenCongestionPeriodMapper.selectGreenDataHist(date.toString());
            LinkedList<AnalysisGreenCongestionPeriod> queue = new LinkedList<>();
            String greenId = "";
            String roadDirection = "";
            String startTime = "";
            String endTime = "";
            for(AnalysisGreenCongestionPeriod temp : greenList){
                if(temp.getGreenId().equals(greenId) && temp.getRoadDirection().equals(roadDirection)){
                    if(temp.getStatus().equals("2")){
                        List<AnalysisGreenCongestionPeriod> list = queue.stream().filter(item -> item.getStatus().equals("3")).collect(Collectors.toList());
                        if(!CollectionUtils.isNotEmpty(list)){
                            List<AnalysisGreenCongestionPeriod> list1 = queue.stream().filter(item -> item.getStatus().equals("2")).collect(Collectors.toList());
                            if(!CollectionUtils.isNotEmpty(list1)){
                                startTime = temp.getStartTime();
                            }
                        }else {
                            if(list.size() == 1 && queue.getFirst().getStatus().equals("3") ){
                                endTime = queue.getFirst().getStartTime();
                                Map<String,Object> map = new HashMap<>();
                                map.put("startTime",startTime);
                                map.put("endTime",endTime);
                                map.put("weekNumber",weekNumber);
                                map.put("weekDay",i + 1);
                                map.put("roadDirection",roadDirection);
                                map.put("peakType",isMoning(startTime));
                                map.put("weekStartTime",todayOfWeek);
                                map.put("weekEndTime",endOfWeek);
                                map.put("greenId",greenId);
                                map.put("status",queue.getFirst().getStatus());
                                //写入缓行数据
                                analysisGreenCongestionPeriodMapper.insertGreenCongestionPeriodData(map);
                                //记录缓行开始时间
                                if(queue.get(1).getStatus().equals("2")) {
                                    startTime = queue.get(1).getStartTime();
                                }else if (queue.getLast().getStatus().equals("2")) {
                                    startTime = queue.getLast().getStartTime();
                                }else {
                                    startTime = temp.getStartTime();
                                }
                            }
                        }
                    }else if(temp.getStatus().equals("1")){
                        List<AnalysisGreenCongestionPeriod> list = queue.stream().filter(item -> item.getStatus().equals("3")).collect(Collectors.toList());
                        if(CollectionUtils.isNotEmpty(list) && queue.size() > 2 && list.size() == 1 && queue.getFirst().getStatus().equals("3")){
                            endTime = queue.getFirst().getStartTime();
                            Map<String,Object> map = new HashMap<>();
                            map.put("startTime",startTime);
                            map.put("endTime",endTime);
                            map.put("weekNumber",weekNumber);
                            map.put("weekDay",i + 1);
                            map.put("roadDirection",roadDirection);
                            map.put("peakType",isMoning(startTime));
                            map.put("weekStartTime",todayOfWeek);
                            map.put("weekEndTime",endOfWeek);
                            map.put("greenId",greenId);
                            map.put("status",queue.getFirst().getStatus());
                            //写入缓行数据
                            analysisGreenCongestionPeriodMapper.insertGreenCongestionPeriodData(map);

                            if(queue.get(1).getStatus().equals("2")) {
                                startTime = queue.get(1).getStartTime();
                            }
                        }else {
                            List<AnalysisGreenCongestionPeriod> list1 = queue.stream().filter(item -> item.getStatus().equals("2")).collect(Collectors.toList());
                            if(CollectionUtils.isNotEmpty(list1) && queue.size() > 2 && list1.size() == 1 && queue.getFirst().getStatus().equals("2") && !queue.get(1).getStatus().equals("3")){
                                endTime = queue.getFirst().getStartTime();
                                Map<String,Object> map = new HashMap<>();
                                map.put("startTime",startTime);
                                map.put("endTime",endTime);
                                map.put("weekNumber",weekNumber);
                                map.put("weekDay",i + 1);
                                map.put("roadDirection",roadDirection);
                                map.put("peakType",isMoning(startTime));
                                map.put("weekStartTime",todayOfWeek);
                                map.put("weekEndTime",endOfWeek);
                                map.put("greenId",greenId);
                                map.put("status",queue.getFirst().getStatus());
                                //写入缓行数据
                                analysisGreenCongestionPeriodMapper.insertGreenCongestionPeriodData(map);
                            }
                        }
                    }else if(temp.getStatus().equals("3")){
                        List<AnalysisGreenCongestionPeriod> list = queue.stream().filter(item -> item.getStatus().equals("3")).collect(Collectors.toList());
                        if(!CollectionUtils.isNotEmpty(list)){
                            List<AnalysisGreenCongestionPeriod> list1 = queue.stream().filter(item -> item.getStatus().equals("2")).collect(Collectors.toList());
                            if(CollectionUtils.isNotEmpty(list1) &&  list1.size() >= 1 && queue.getLast().getStatus().equals("2") ){
                                endTime = queue.getLast().getStartTime();
                                Map<String,Object> map = new HashMap<>();
                                map.put("startTime",startTime);
                                map.put("endTime",endTime);
                                map.put("weekNumber",weekNumber);
                                map.put("weekDay",i + 1);
                                map.put("roadDirection",roadDirection);
                                map.put("peakType",isMoning(startTime));
                                map.put("weekStartTime",todayOfWeek);
                                map.put("weekEndTime",endOfWeek);
                                map.put("greenId",greenId);
                                map.put("status",queue.getLast().getStatus());
                                //写入缓行数据
                                analysisGreenCongestionPeriodMapper.insertGreenCongestionPeriodData(map);
                                //记录拥堵开始时间
                                startTime = temp.getStartTime();
                            }else {
                                startTime = temp.getStartTime();
                            }
                        }
                    }
                    enqueue(queue,temp);
                }else if(temp.getGreenId().equals(greenId) && !temp.getRoadDirection().equals(roadDirection)){
                    queue.clear();
                    roadDirection = temp.getRoadDirection();
                    if(temp.getStatus().equals("2")){
                        startTime = temp.getStartTime();
                    }
                    enqueue(queue,temp);
                }else {
                    queue.clear();
                    greenId = temp.getGreenId();
                    roadDirection = temp.getRoadDirection();
                    if(temp.getStatus().equals("2")){
                        startTime = temp.getStartTime();
                    }
                    enqueue(queue,temp);
                }
            }

        }
    }

    public void enqueue(LinkedList<AnalysisGreenCongestionPeriod> queue , AnalysisGreenCongestionPeriod temp ){
        if(queue.size() >= 3 ){
            queue.poll();
        }
        queue.offer(temp);
    }

    public Integer isMoning(String date) throws ParseException {
        DateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date temp = fmt.parse(date);
        int hour = temp.getHours();
        if(hour <= 12){
            return 1;
        }else {
            return 2;
        }
    }
}
